require 'spec_helper'

describe GrisPaginator::PaginationHelpers do
  let(:endpoint) do
    Grape::Endpoint.new(
      Grape::Util::InheritableSetting.new, path: '/', method: :get, root: false
    )
  end
  let(:total_count) { 20 }

  context 'paginate' do
    before do
      total_count.times do |index|
        Beach.create! name: "Playa Grande #{index}"
      end
      allow(endpoint).to receive(:env).and_return({})
    end

    it 'raises an ArgumentError without a presenter' do
      expect { endpoint.paginate(Beach) }.to raise_error ArgumentError
    end

    it 'returns an ActiveRecord_Relation that responds to total_count' do
      allow(endpoint).to receive(:params).and_return({})
      result = endpoint.paginate(Beach, with: BeachesPresenter)
      expect(result.class).to be Beach::ActiveRecord_Relation
      expect(result.total_count).to eq total_count
    end

    context 'size' do
      it 'limits the result set count based on params[:size]' do
        per_page = 10
        allow(endpoint).to receive(:params).and_return(size: per_page)
        result = endpoint.paginate Beach, with: BeachesPresenter
        expect(result.size).to eq per_page
        expect(result.current_page).to eq 1
        expect(result.total_pages).to eq total_count / per_page
        expect(result.total_count).to eq total_count
      end
    end

    context 'page' do
      it 'jumps to the page of results corresponding to params[:page]' do
        per_page = 2
        page = 2
        allow(endpoint).to receive(:params)
          .and_return(size: per_page, page: page)
        result = endpoint.paginate Beach, with: BeachesPresenter
        expect(result.size).to eq per_page
        expect(result.first.name).to eq 'Playa Grande 2'
        expect(result.current_page).to eq page
        expect(result.total_pages).to eq total_count / per_page
        expect(result.total_count).to eq total_count
      end
    end

    context 'conditions' do
      it 'filters collection based on conditions argument' do
        allow(endpoint).to receive(:params).and_return({})
        result = endpoint.paginate Beach,
                                   conditions: { name: 'Playa Grande 1' },
                                   with: BeachesPresenter
        expect(result.size).to eq 1
        expect(result.first.id).to eq 2
        expect(result.first.name).to eq 'Playa Grande 1'
        expect(result.current_page).to eq 1
        expect(result.total_pages).to eq 1
        expect(result.total_count).to eq 1
      end
    end

    context 'sort' do
      it 'orders collection by default without sort argument or sort params' do
        allow(endpoint).to receive(:params).and_return({})
        result = endpoint.paginate Beach, with: BeachesPresenter
        expect(result.first.id).to eq Beach.first.id
      end

      it 'orders collection based on sort argument' do
        allow(endpoint).to receive(:params).and_return({})
        result = endpoint.paginate Beach, sort: 'id DESC',
                                          with: BeachesPresenter
        expect(result.first.id).to eq Beach.last.id
      end

      it 'orders collection based on sort params' do
        allow(endpoint).to receive(:params).and_return(sort: 'id DESC')
        result = endpoint.paginate Beach, with: BeachesPresenter
        expect(result.first.id).to eq Beach.last.id
      end
    end
  end
end
